@@ -20,7 +20,7 @@ mvrnorm <- function(n = 1, mu, Sigma, tol=1e-6, empirical = FALSE)
     if(!all(dim(Sigma) == c(p,p))) stop("incompatible arguments")
     eS <- eigen(Sigma, symmetric = TRUE, EISPACK = TRUE)
     ev <- eS$values
-    if(!all(ev >= -tol*abs(ev[1]))) stop("'Sigma' is not positive definite")
+    if(!all(ev >= -tol*abs(ev[1L]))) stop("'Sigma' is not positive definite")
     X <- matrix(rnorm(p * n), n)
     if(empirical) {
         X <- scale(X, TRUE, FALSE) # remove means
@@ -29,7 +29,7 @@ mvrnorm <- function(n = 1, mu, Sigma, tol=1e-6, empirical = FALSE)
     }
     X <- drop(mu) + eS$vectors %*% diag(sqrt(pmax(ev, 0)), p) %*% t(X)
     nm <- names(mu)
-    if(is.null(nm) && !is.null(dn <- dimnames(Sigma))) nm <- dn[[1]]
+    if(is.null(nm) && !is.null(dn <- dimnames(Sigma))) nm <- dn[[1L]]
     dimnames(X) <- list(nm, NULL)
     if(n == 1) drop(X) else t(X)
 }
